#!/bin/bash
#
# Example of what can be done with the fifo, to enable news fetching
# transparently and on demand.  This is just an example!  Not for
# production use!
#

function fail () { echo "$@" >&2; exit 1 }

[ "$UID" = "0" ] && fail "Cannot run as root"

[ -p /var/spool/sn/.fifo ] || fail "/var/spool/sn/.fifo isn't a fifo"

cd /var/spool/sn || fail "Can't cd to /var/spool/sn"

# This is an example script that does news-on-demand.
# Do not run this script in a production environment.
# Press Control-C at any time to kill this script.
#
# Entering loop, waiting to read newsgroup names...

while : ; do
  group=
  read group </var/spool/sn/.fifo
  [ -z "$group" ] && continue

  # fifo says someone is reading group $group.

  #
  # We don't want to accidentally eval a funny shell command.
  #
  case "$group" in
  *' '*) continue;;
  *'='*) continue;;
  *'"'*) continue;;
  *"'"*) continue;;
  *'`'*) continue;;
  *'/'*) continue;;
  *'<'*) continue;;
  *'>'*) continue;;
  [abcdefghijklmnopqrstuvwxyz]*.*[abcdefghijklmnopqrstuvwxyz]);;
  *) continue;;
  esac

  # Group $group is a valid group name,

  if [ ! -d "$group" -a -e "$group/.outgoing" ]; then
    # ... but we are not subscribed to it.  Carry on.
    continue
  fi

  # ... and we are subscribed to it,

  if [ ! -e "$group/.outgoing" ]; then
    # ... but it is a private group.  Carry on.
    # An alternative strategy might be to create the newsgroup on
    # the fly.
    continue
  fi

  # ... but are we already downloading this group?

  #
  # We'll turn the newsgroup name into a shell variable that will
  # later contain the pid of this get process.
  # An alternative strategy might be to create a lockfile.
  #
  group=`echo $group |tr '.' '_'`

  pid=`eval "echo \\$$group"`
  if [ "$pid" ]; then
    if kill -0 $pid 2>/dev/null; then
      # Yes, a previous get on $group is already running as pid $pid,
      # so we will not start a new get.
      continue
    else
      # No, the previous get on group $group has completed,
      # so we will start a new one.
      eval "$group="        
    fi
  else
    # No, this is a new request for $group.
  fi

  # Running news fetcher on group $group.  We are not concerned
  # if the fetcher should fail.
  # We might check if the host is reachable first and start pppd if
  # not, or we might run diald.

  /usr/lib/sn/snget `echo $group |tr '_' '.'` &
  eval "$group=$!"  # Save the pid
done

